"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ms_1 = __importDefault(require("ms"));
const qs_1 = require("qs");
const utils_1 = require("@poppinss/utils");
const Route_1 = require("./Route");
const Store_1 = require("./Store");
const Group_1 = require("./Group");
const BriskRoute_1 = require("./BriskRoute");
const Resource_1 = require("./Resource");
const helpers_1 = require("../helpers");
class Router {
    constructor(_encryption, _routeProcessor) {
        this._encryption = _encryption;
        this._routeProcessor = _routeProcessor;
        this.routes = [];
        this.BriskRoute = BriskRoute_1.BriskRoute;
        this.RouteGroup = Group_1.RouteGroup;
        this.RouteResource = Resource_1.RouteResource;
        this.Route = Route_1.Route;
        this._matchers = {};
        this._store = new Store_1.Store();
        this._lookupStore = [];
        this._openedGroups = [];
        this._testRoutePatternCounter = 0;
        this._testsHandler = async () => {
            return 'handled by tests handler';
        };
    }
    _getRecentGroup() {
        return this._openedGroups[this._openedGroups.length - 1];
    }
    route(pattern, methods, handler) {
        const route = new Route_1.Route(pattern, methods, handler, this._matchers);
        const openedGroup = this._getRecentGroup();
        if (openedGroup) {
            openedGroup.routes.push(route);
        }
        else {
            this.routes.push(route);
        }
        return route;
    }
    any(pattern, handler) {
        return this.route(pattern, ['HEAD', 'OPTIONS', 'GET', 'POST', 'PUT', 'PATCH', 'DELETE'], handler);
    }
    get(pattern, handler) {
        return this.route(pattern, ['GET'], handler);
    }
    post(pattern, handler) {
        return this.route(pattern, ['POST'], handler);
    }
    put(pattern, handler) {
        return this.route(pattern, ['PUT'], handler);
    }
    patch(pattern, handler) {
        return this.route(pattern, ['PATCH'], handler);
    }
    destroy(pattern, handler) {
        return this.route(pattern, ['DELETE'], handler);
    }
    group(callback) {
        const group = new Group_1.RouteGroup([]);
        const openedGroup = this._getRecentGroup();
        if (openedGroup) {
            openedGroup.routes.push(group);
        }
        else {
            this.routes.push(group);
        }
        this._openedGroups.push(group);
        callback();
        this._openedGroups.pop();
        return group;
    }
    resource(resource, controller) {
        const resourceInstance = new Resource_1.RouteResource(resource, controller, this._matchers);
        const openedGroup = this._getRecentGroup();
        if (openedGroup) {
            openedGroup.routes.push(resourceInstance);
        }
        else {
            this.routes.push(resourceInstance);
        }
        return resourceInstance;
    }
    shallowResource(resource, controller) {
        const resourceInstance = new Resource_1.RouteResource(resource, controller, this._matchers, true);
        const openedGroup = this._getRecentGroup();
        if (openedGroup) {
            openedGroup.routes.push(resourceInstance);
        }
        else {
            this.routes.push(resourceInstance);
        }
        return resourceInstance;
    }
    on(pattern) {
        const briskRoute = new BriskRoute_1.BriskRoute(pattern, this._matchers);
        const openedGroup = this._getRecentGroup();
        if (openedGroup) {
            openedGroup.routes.push(briskRoute);
        }
        else {
            this.routes.push(briskRoute);
        }
        return briskRoute;
    }
    where(param, matcher) {
        this._matchers[param] = typeof (matcher) === 'string' ? new RegExp(matcher) : matcher;
        return this;
    }
    toJSON() {
        return helpers_1.toRoutesJSON(this.routes);
    }
    commit() {
        const names = [];
        this.toJSON().forEach((route) => {
            if (route.name && names.indexOf(route.name) > -1) {
                throw new utils_1.Exception(`Duplicate route name \`${route.name}\``, 500, 'E_DUPLICATE_ROUTE_NAME');
            }
            if (route.name) {
                names.push(route.name);
            }
            if (this._routeProcessor) {
                this._routeProcessor(route);
            }
            this._lookupStore.push({
                handler: route.handler,
                name: route.name,
                pattern: route.pattern,
                domain: route.domain || 'root',
            });
            this._store.add(route);
        });
        this.routes = [];
        this._matchers = {};
    }
    match(url, method, domain) {
        const matchingDomain = domain ? this._store.matchDomain(domain) : [];
        if (!matchingDomain.length) {
            return this._store.match(url, method);
        }
        return this._store.match(url, method, {
            storeMatch: matchingDomain,
            value: domain,
        });
    }
    lookup(routeIdentifier, forDomain) {
        return this._lookupStore.find(({ name, pattern, handler, domain }) => {
            return [name, pattern, handler].indexOf(routeIdentifier) > -1 && (!forDomain || forDomain === domain);
        }) || null;
    }
    makeUrl(routeIdentifier, options, domain) {
        const route = this.lookup(routeIdentifier, domain);
        if (!route) {
            return null;
        }
        options = Object.assign({ qs: {}, params: {}, domainParams: {}, prefixDomain: true }, options);
        let url = helpers_1.processPattern(route.pattern, options.params);
        const qs = qs_1.stringify(options.qs);
        if (qs) {
            url = `${url}?${qs}`;
        }
        return route.domain !== 'root' && options.prefixDomain
            ? `//${helpers_1.processPattern(route.domain, options.domainParams)}${url}`
            : url;
    }
    makeSignedUrl(routeIdentifier, options, domain) {
        const route = this.lookup(routeIdentifier, domain);
        if (!route) {
            return null;
        }
        options = Object.assign({ qs: {}, params: {}, domainParams: {}, prefixDomain: true }, options);
        const expiresIn = options['expiresIn'];
        if (expiresIn) {
            const milliseconds = typeof (expiresIn) === 'string' ? ms_1.default(expiresIn) : expiresIn;
            options.qs.expires_at = Date.now() + milliseconds;
        }
        const signature = this._encryption
            .child({ hmac: false })
            .encrypt(this.makeUrl(route.pattern, { qs: options.qs, params: options.params, prefixDomain: false }));
        options.qs.signature = signature;
        return this.makeUrl(route.pattern, options);
    }
    forTesting(pattern, methods, handler) {
        pattern = pattern || `_test_${this._testRoutePatternCounter++}`;
        methods = methods || ['GET'];
        handler = handler || this._testsHandler;
        const route = this.route(pattern, methods, handler);
        this.commit();
        return route;
    }
}
exports.Router = Router;
